/* program: GBtPov1r.prg
   Author:  G.F. Barrett and S.G. Donald, 2009, Gini Inference, JBES
   Purpose: Apply the Bootstrap-t Gini inference procedures to real
                data. This program reads in a (weighted) samples
                and constructed the 95% Confidence Interval estimates
                for the Gini poverty indices.

   Revision 2.0 March 2008

Data Files

load x[5116,2]=e:\data\lorenz\cps784fr.dat;
load x[5616,2]=e:\data\lorenz\cps884fr.dat;
load x[5393,2]=e:\data\lorenz\cps984fr.dat;

Column 1 = welfare measure
Column 2 = record weight

	Last change: GB 24/10/2008 2:36:58 PM
*/


/* ========================================================= */
/* Set Global Variables in this section                      */
/* ========================================================= */

output file=g:\Ginis\revised\pubcode\CPS88Btp.out reset;
load x[5616,2]=g:\Ginis\revised\pubcode\cps884fr.dat;
xnam="CPS1988";

Perc=1;  @ Set Perc=1 for Percentile method @
         @ Set Perc~=1 for Efrons Hybrid Method @

rndseed 1364;  /* set the seed */
brep1=499;     /* # of bootstrap iterations  */
kb=0.95;       /* width of the CI  */
zz=8480;       /* the poverty line */

print "poverty line = " zz;
/* ========================================================= */

/* Normalise the record weight so the total = 1  */
x[.,2]=x[.,2]/sumc(x[.,2]);

if (Perc.eq 1);
    Method="Percentile Method";                            
else;                                                      
    Method="Hybrid Method";                                
endif;                                                     

ds=datestr(0);ts=timestr(0);
print "Date " ds "   " ts ;

nx=rows(x);         /* # of observations for sample x */

  dd=zeros(10,1);
  dd[1]=1.00;
  dd[2]=1.25;
  dd[3]=1.5;
  dd[4]=2.0;
  dd[5]=3.0;
  dd[6]=4.0;
  dd[7]=5.00;
  dd[8]=10.00;
  dd[9]=20.00;
  dd[10]=40.0;

@ Set the values of the EGini inequality aversion parameters @
  aa=zeros(10,1);
  aa[1]=1.0;
  aa[2]=2.0;
  aa[3]=3.0;
  aa[4]=4.0;
  aa[5]=5.0;
  aa[6]=6.0;
  aa[7]=8.00;
  aa[8]=10.00;
  aa[9]=20.00;
  aa[10]=40.0;

CIx=zeros(20,3); /* rows are SGR,A,W,EGR,A,W;   columns are point est., lower bound, then upper bound */
Ix=zeros(20,1);

et = hsec;       /* record the time length of this run */

@ output off; @

@ Call the Gini procedure for x and y @
                                          
{Ix,Ixv}=GPov(x,zz,nx);

CIx[.,1]=Ix;

@ Construct the test stat then bootstrap it @
                                          
                                          
Id = (Ix);
tp = ((Id).*(Ixv.^(-0.5)));
sighat=Ixv.^(0.5);                        
@ Now Bootstrap the pivotal test stat @   
@ thenCalculate the Boot-t CI         @

{CIx[.,2:3]}=BStest(x,Id,sighat,tp,brep1,kb,nx,zz);

 sst=zeros(3,1);
{sst}=sumst(x,zz,nx);

output on;
et = hsec - et;
ds=datestr(0);ts=timestr(0);

print "==============================================================";
print "Results for Consistent Gini Poverty Indices with Boot-t Tests";
print "Date " ds "   " ts ;
print "Time, in seconds" et/100;
print "Sample size = " nx;
print "Bootstrap repetitions = " brep1;
print "  ";
print "Bootstrap CI Estimation Results for Sample: " xnam;
print "CI width == " kb;
print Method;
print "Head-count ratio F(z) = " sst[1];
print "Sample mean           = " sst[2];
print "Mean income for poor  = " sst[3];
print "==============================================================";
print "  ";
print "95% CI Estimation Results for Sample: " xnam;
print "Poverty line = " zz;
print "------------------------------------------------------------------";
print "Index                  Point Est          Lower Bd       Upper Bd";
print "------------------------------------------------------------------";
print "S-Gini Poverty Indices";
print dd~CIx[1:10,.];
print " ";
print "E-Gini Poverty Indices";
print aa~CIx[11:20,.];
print " ";

output off;
end;

  /*====================== Proc: GPov(x,z,n) =========================*/
proc (2)=GPov(x,z,n);

  local nz,nnz,xdot,ndot,ndz,ndz1,xnz1,Fz,meanz,zdot,LCzx,nrz,mnx,SGP,SGPvz,SGPse,
        EGP,EGPvz,EGPse;


@ Normalise the total weight to 1.0 @
x[.,2]=x[.,2]/sumc(x[.,2]);
n=rows(x);
nz=sumc( (x[.,1].le z));
nnz=n-nz;
@ Construct the vector of unique xs, sorted in ascending order @

{xdot}=stuniq(x);
ndot=rows(xdot);

ndz=sumc( (xdot[.,1].le z));
ndz1=(ndz.lt ndot)*(ndz+1)+ (ndz.eq ndot)*(ndz);
xnz1=xdot[ndz1,1];

Fz=sumc( (xdot[.,1].le z).*xdot[.,2]);

meanz=sumc( (xdot[.,1].le z).*xdot[.,1].*xdot[.,2])/sumc((xdot[.,1].le z).*xdot[.,2]);
zdot=xdot[1:ndz,.];

@ Calculate the sample mean   @
mnx = sumc(x[.,1].*x[.,2]);

@ Calculate the SGini poverty index @
 {SGP} = SGiniP(zdot,meanz,Fz,z);

@ Calculate the Asymptotic Variance of the SGini welfare indices @
 {SGPVz} = SGPVar(zdot,meanz,Fz,z,nnz,n);
  SGPse=(SGPVz/n);

@ Calculate the EGini poverty index @
{EGP} = EGiniP(zdot,meanz,Fz,z);

@ Calculate the Asymptotic Variance of the EGini indices @
 {EGPVz} = EGPVar(zdot,meanz,Fz,z,xnz1,mnx,nnz,n);
  EGPse=(EGPVz/n);

@ NOTE - returning Variances NOT se @

retp(SGP|EGP,SGPse|EGPse);
endp;



 /*====================== Proc: Stuniq(x) =========================*/

proc (1)=Stuniq(x);

  local j,zd,pd,xd;

  zd = unique(x[.,1],1);     /* a vector consisting of the distinct x values */
  pd = rows(zd);           /* number of distinct combined x */

  xd = zeros(pd,2);        /* initialize the vector */
  xd[.,1]=zd;
  for j (1,pd,1);          /* construct the xdot vector */
      xd[j,2] = sumc((x[.,1] .eq zd[j]).* x[.,2]);
  endfor;

retp(xd);
endp;

 /*====================== Proc: Sumst(x,z,) =========================*/
proc (1)=Sumst(x,z,n);

  local nz,nnz,xdot,ndot,ndz,ndz1,xnz1,Fz,meanz,zdot,LCzx,nrz,mnx,SGP,SGPvz,SGPse,
        EGP,EGPvz,EGPse;


@ Normalise the total weight to 1.0 @
x[.,2]=x[.,2]/sumc(x[.,2]);
n=rows(x);
nz=sumc( (x[.,1].le z));
nnz=n-nz;
@ Construct the vector of unique xs, sorted in ascending order @

{xdot}=stuniq(x);
ndot=rows(xdot);

ndz=sumc( (xdot[.,1].le z));
ndz1=(ndz.lt ndot)*(ndz+1)+ (ndz.eq ndot)*(ndz);
xnz1=xdot[ndz1,1];

Fz=sumc( (xdot[.,1].le z).*xdot[.,2]);

meanz=sumc( (xdot[.,1].le z).*xdot[.,1].*xdot[.,2])/sumc((xdot[.,1].le z).*xdot[.,2]);
zdot=xdot[1:ndz,.];

@ Calculate the sample mean   @
mnx = sumc(x[.,1].*x[.,2]);

retp(Fz|mnx|meanz);
endp;

/*====================== Proc: SGiniP(zdot,meanz,Fz,z) =========================*/

proc (1)=SGiniP(x,mn,Fzz,zz);

  local swfi,swfm,nd,swn,swn1,P;

     nd=rows(x);
     swn=cumsumc(x[.,2]);
     swn=Fzz-swn;
     swn[nd]=0;  @ due to machine precision - this may be an extremely small negative number, reset to 0 @
     swn1=Fzz|swn[1:nd-1];

     swfm=sumc((swn1^dd'-swn^dd').*x[.,1])./(Fzz^dd);
     P=Fzz*(1-(swfm/zz));
     retp(P);
endp;

/*====================== Proc: SGPVar(zdot,meanz,Fz,z,nnz,n) =========================*/

proc (1)=SGPVar(x,mn,Fzz,zz,nnzz,n);

  local nd,pj,pjm1,GLx,GLx1,Inflf,Inflnp,I1,I2,I3,I4,I5,d4j,i,varapg,VarP;

     nd=rows(x);
     pj=cumsumc(x[.,2]);
     pjm1=0|pj[1:nd-1];

     GLx=cumsumc(x[.,1].*x[.,2]);
     GLx1=0|GLx[1:nd-1,1];

     I1=sumc(x[.,1].* (-1*dd'.*( pj.*(Fzz-pj)^(dd-1)' - pjm1.*(Fzz-pjm1)^(dd-1)' )-((Fzz-pj)^dd' - (Fzz-pjm1)^dd') )  )./(Fzz^dd);
     I2=Fzz*I1;
     I3=sumc(x[.,1].*((Fzz-pj)^dd'-(Fzz-pjm1)^dd')+(dd'.*( (Fzz-pj)^(dd-1)'.*GLx-(Fzz-pjm1)^(dd-1)'.*GLx1 ))  )./(Fzz^(dd-1));

     I1=I1';I2=I2';I3=I3';

     I4=zeros(nd,rows(dd));
     I5=zeros(nd,rows(dd));

     d4j=  (-1*dd'.*( (Fzz-pj)^(dd-1)'-(Fzz-pjm1)^(dd-1)' ));

     Inflf=ones(nd,1)-Fzz ;

     for i (1,nd,1);
        I4[i,.]=((-1/Fzz^(dd-1)).*sumc( ((x[i,1].lt x[.,1]).*x[.,1]).*d4j))';
        I5[i,.]= ((x[i,1]/Fzz^(dd-1)).*sumc( (x[i,1].lt x[.,1]).*d4j))';
     endfor;

       Inflnp=-Fzz+Fzz*I1/zz-(I2+I3)/zz;

 @ The following is for the case where delta / aa ==1, when the SGini poverty index @
 @ is the average income gap                                                        @


    varapg= (sumc(  ( ( (1-x[.,1]/zz)-(1-mn/zz)*Fzz)^2) .*x[.,2])+(1-Fzz)*((1-mn/zz)*Fzz).^2.0 )/1;

@ Note that the non-poor observations contribute to the influence function for the poverty index @
@ through the term F(z) .... hence the second term in the expression below @

     VarP= (sumc(  (Inflf - Inflf*I1/zz-(I2+I3+I4+I5)/zz)^2.0 .*x[.,2])  + (1-Fzz)*(Inflnp').^2.0 )/1;

     VarP[1]=varapg;

retp(VarP);
endp;
/*====================== Proc: EGiniP(zdot,meanz,Fz,z) =========================*/

proc (1)=EGiniP(x,mnz,Fzz,zz);

@ poverty index calculation double-checked - it is absolutely fine @

  local nd,pj,pj1,TE,aj,bj,bj1,cj,aa1,na,e23;
     x[.,2]=x[.,2];         @ *Fzz  @
     nd=rows(x);
     pj=cumsumc(x[.,2]);
     pj1=zeros(1,1)|pj[1:nd-1];

     aa1=aa+1;

     bj=mnz-x[.,1];
     bj1=bj+(bj.eq 0);
     aj =-1*( x[.,1].* (cumsumc(x[.,2])) - cumsumc(x[.,1].*x[.,2]) );
     
     cj= (bj.ne 0).*(((bj.*pj-aj)^aa1') - ((bj.*pj1-aj)^aa1'))./(bj1) - (bj.eq 0).* aj.*(pj-pj1)*aa1';
         
     TE= (aa1)^(-1).*sumc(cj)/Fzz;
            
     retp(Fzz - mnz*Fzz/(zz)+TE^(1/aa)/(zz));
endp;

/*====================== Proc: EGPVar(zdot,meanz,Fz,z,ynz1,mnx,nnz,n) =========================*/
proc (1)=EGPVar(x,mn,Fzz,zz,ynz1,mnx,nnzz,n);

  local nd,na,pj,pj1,GLx,i,j,TE,I1,aj,anz1,bj,bj1,bj0,bj10,bpa,cj,aa1,aa12,
    e1,e2,e3,e4,e5,e22,e23,e24,e25,g1,g2,h1,h2,VarP,Infli,Inflinp,
    ie1,ie2,ie3,ie4,h1np,h2np,Ie27np,TE2,TE1,ie0,e10,ie4n;

     nd=rows(x);
     pj=cumsumc(x[.,2]);
     pj1=zeros(1,1)|pj[1:nd-1];
     aa1=aa+1;   aa12=2*ones(10,1);
     na=rows(aa);

     bj=mn-x[.,1];  @ A Test thing @
     
     bj1=bj+(bj.eq 0);
     
     aj =-1*( x[.,1].* (cumsumc(x[.,2])) - cumsumc(x[.,1].*(x[.,2])) );
     anz1=Fzz*mn;
     
     bpa=(bj.*pj-aj);

    cj= (bj.ne 0).*(((bj.*pj-aj)^aa1') - ((bj.*pj1-aj)^aa1'))./ (bj1) - (bj.eq 0).* aj.*(pj-pj1)*aa1';
    TE=(aa1)^(-1).*sumc(cj)/Fzz;
         

     e1= ((pj.*(bj.*pj-aj)^aa') - pj1.*((bj.*pj1-aj)^aa'))./ (bj1) ;
     e2= (((bj.*pj-aj)^aa1') - ((bj.*pj1-aj)^aa1'))./ ((bj1^2)*aa1') ;
     e3= (-1*aj)^(aa-1)'.* ((pj^2-pj1^2)*aa')/2;
     e4= (((bj.*pj-aj)^aa') - ((bj.*pj1-aj)^aa'))./ (bj1) ;
     e5=  (-1*aj)^(aa-1)'.*((pj-pj1)*aa');   
   
     
     I1= -1/(zz)*( (Fzz*zz-anz1)- (zz-x[.,1]) );

     g1= (bj.ne 0).*(e1-e2)+(bj.eq 0).*e3;
     g2= (bj.ne 0).*e4+(bj.eq 0).*e5;
     
     ie0=sumc(g1)/(Fzz^2);
     ie1=sumc(x[.,1].*g1)/(Fzz^2);
     ie2=Fzz*ie1;
     ie3=ie2+sumc(aj[.,1].*g2)/Fzz;

     Infli=zeros(nd,rows(aa));

   for i (1,nd,1);
        h1=(x[i,1].le x[.,1]).*(x[.,1]-x[i,1]);
        ie4=sumc(h1.*g2)/Fzz;


        Infli[i,.]=(ie0.*(zz*(1-Fzz)+((zz*Fzz-anz1)-(zz-x[i,1])))
         -((1-Fzz).*ie1+ie2-ie3-ie4))';

        Infli[i,.]=-((Fzz*zz-anz1)-(zz-x[i,1]))/zz
               +((1./aa).*TE^((1./aa)-1))'.*Infli[i,.]/zz;
    endfor;
        
 @ Influence function contribution of non-poor observations @
       
       Inflinp=ie0*(zz*(0-Fzz)+((zz*Fzz-anz1)))
         -((0-Fzz)*ie1+ie2-ie3);

      Inflinp=-(Fzz*zz-anz1)/zz
            +(1./aa).*TE^((1./aa)-1).*Inflinp/zz;

 VarP= (sumc(Infli^2.0))/n + ((nnzz*(Inflinp.^2)) )/n;

     retp(VarP);
endp;
/*==============================================================================*/


/*====================== Proc: BStest(x,Ix,sigh,tp,brep,k,n,z) =========================*/
 /* This procedure bootstraps the asymptotic pivotal test stat      */

proc (1)=BStest(xx,Id,sigh,tp,breps,kbt,n,z);

     local ib,xs,ux,Ixs,Ixvs,tpb,Pvals,mnxs,varmnxs,i,j,kb,m, Ubdi,
     Lbdi,Ubd,Lbd, kbb ;

Pvals=zeros(20,1);

@ Initialise the vectors to accumulate the Pval contribution from each iteration @

Ixs=zeros(1,20);
Ixvs=zeros(1,20);
tpb=zeros(breps+1,20);

Ubd=zeros(20,1);
Lbd=zeros(20,1);
m=zeros(1,1);


for ib (1,breps,1);
/* initialize the vector corresponding to the resampled x */
   xs=zeros(n,2);
/* resample the x distribution with replacement */
   ux=floor(n*rndu(n,1)+ones(n,1));
   xs[.,1]=xx[ux,1];
   xs[.,2]=xx[ux,2];
   xs[.,2]=xs[.,2]/sumc(xs[.,2]);   @ normalise weights to sum to 1 @


@ Call the Gini procedure for resampled x  @

{Ixs,Ixvs}=GPov(xs,z,n);


@ Perform 1-tailed test of null of equality @

tpb[ib,.]= ((Ixs-Id).*((Ixvs).^(-0.5)))';

endfor;

tpb[breps+1,.]=tp';



kbb=kbt*(breps+1);
m=(breps+1)*( (1.0-kbt)/2);
                                         
Ubdi=minc((breps+1)|(breps+1-m));
Lbdi=maxc(1|m);                          



for i (1,20,1);
        tpb[.,i]=sortc(tpb[.,i],1);


      if (Perc.ne 1);

        @ HI. Hybrid Method: Barr(1994), Efron(1990) @
             Ubd[i]=Id[i]+tpb[Ubdi,i]*sigh[i];
             Lbd[i]=Id[i]+tpb[Lbdi,i]*sigh[i];

       else;

           Ubd[i]=Id[i]-tpb[Lbdi,i]*sigh[i];
           Lbd[i]=Id[i]-tpb[Ubdi,i]*sigh[i];

       endif;

endfor;



retp(Lbd~Ubd);
endp;
/*==============================================================================*/
